
# generado automaticamente

import os
import sys
from dotenv import load_dotenv

if "--dev" in sys.argv:
    filenv = "../.env.dev"
    DEBUG = True
else:
    filenv = "../.env"
    DEBUG = False

dir_path = os.path.dirname(os.path.realpath(__file__))
load_dotenv(dotenv_path=os.path.join(dir_path, filenv), verbose=True, override=True)

def parse_url_mysql(user, password, host, port, name):
    return f"mysql+pymysql://{user}:{password}@{host}:{port or 3306}/{name}?charset=UTF8MB4&autocommit=false"


TESTING = False
JWT_SECRET_KEY = os.getenv("JWT_SECRET_KEY", 'development')

NAME     = os.getenv("NAME", os.getenv("HOSTNAME"))
HOST     = os.getenv("HOST", "localhost")
PORT     = os.getenv("PORT", 5000)
REGISTER = os.getenv("REGISTER", "http://ip_node")
MS_HOST = os.getenv("MS_HOST")

MYSQL_NAME = os.getenv("MYSQL_NAME")
MYSQL_USER = os.getenv("MYSQL_USER")
MYSQL_PASSWORD = os.getenv("MYSQL_PASSWORD")
MYSQL_HOST = os.getenv("MYSQL_HOST")
MYSQL_PORT = os.getenv("MYSQL_PORT")

SQLALCHEMY_DATABASE_URI = os.getenv("SQLALCHEMY_DATABASE_URI", parse_url_mysql(
    name=MYSQL_NAME,
    user=MYSQL_USER,
    password=MYSQL_PASSWORD,
    port=MYSQL_PORT,
    host=MYSQL_HOST
))
SQLALCHEMY_TRACK_MODIFICATIONS = False
SQLALCHEMY_ENGINE_OPTIONS = {
    "pool_recycle": 490,
    "pool_pre_ping": 60,
}

INSTALLED_APPS = [
    "app.main",
]




from app import db, migrate
from .models import ExampleModel

'''
    bws migrate main__create_table
'''

@migrate.command
def main__create_table():
    return ExampleModel.__table__.create(db.get_engine())




template = """
MYSQL_NAME=ms-name
MYSQL_USER=ms-user
MYSQL_PASSWORD=ms-pass
MYSQL_HOST=127.0.0.1
MYSQL_PORT=3306

JWT_SECRET_KEY=%(secret)s

REGISTER = http://ip_node:5555/ms/register
MS_HOST = https://ip_ms:5000

HOST = "localhost"
PORT = 5000
"""




# generado automaticamente
import logging

logging.basicConfig(
    level=logging.INFO,
    format='[%(levelname).1s %(asctime).19s] %(message)s',
    datefmt='%y-%m-%d %H:%M:%S'
)

from app import app
from bws.admin import create_admin
from bws.admin.bootstrap import bootstrap


logger = logging.getLogger("Boot Admin")

admin = create_admin(app)
bootstrap(app, admin, ["config", "api"])



if __name__ == '__main__':    
    app.run(host=app.config["HOST"], host=app.config["PORT"])



# from app import admin
import importlib
import logging

logger = logging.getLogger("Boot Admin")


def bootstrap(app, admin=None, types=["config", "api"]):    
    # Register admin of apps
    for type_ in types:
        for package in app.config["INSTALLED_APPS"]:
            if type_ == "config":
                assert not admin is None 
                for file in ["config"]:
                    package_admin = "%s.%s" % (package, file)
                    try:
                        logger.info("import %s" % package_admin)
                        module = importlib.import_module(package_admin)
                        register_frontend_views = getattr(module, "register_frontend_views", lambda a: 0)
                        register_api_views = getattr(module, "register_api_views", lambda a: 0)
                        register_frontend_views(admin)
                        register_api_views(admin)
                        logger.info("imported %s" % package_admin)
                    except ModuleNotFoundError as exc:
                        if not package_admin in str(exc):
                            raise exc
            elif type_ == "api":
                for file in ["api"]:
                    package_admin = "%s.%s" % (package, file)
                    try:
                        logger.info("api.import %s" % package_admin)
                        __import__(package_admin)
                        logger.info("api.imported %s" % package_admin)
                    except ModuleNotFoundError as exc:
                        if not package_admin in str(exc):
                            raise exc
                
                app.register_blueprint(importlib.import_module("app").api, url_prefix = "/api")
            
            else:
                for file in [type_]:
                    package_admin = "%s.%s" % (package, file)
                    try:
                        __import__(package_admin)
                        logger.debug("import %s" % package_admin)
                    except ModuleNotFoundError as exc:
                        if not package_admin in str(exc):
                            raise exc



template = """
#!/bin/bash 
#-xe
export DIR_VENV=$PWD/_venv

if [ -d "$DIR_VENV" ]; then
  source $DIR_VENV/bin/activate
fi

cmd_install() {
  # installar. Tambien se puede usar para actualizar agregadas librerias
  if [ ! -d "$DIR_VENV" ]; then
    # sudo apt-get install python3-venv -y
    python3 -m venv $DIR_VENV
  fi
  $DIR_VENV/bin/pip install -r requirements.txt
}

cmd_web() { 
  python3 web.py 
}

cmd_webdev() { 
  python3 web.py --dev
}

cmd_cmd() { 
  python3 cmd.py "$@" 
}

cmd_migrate() { 
  python3 migrate.py "$@" 
}

cmd_pip() { 
  pip "$@" 
}

cmd_pos-commit() { 
    echo "pos-commit";
}

cmd_version() {
  python --version
  pip --version
}

cd "$(dirname "$0")"; _cmd="${1?"cmd is required"}"; shift; "cmd_${_cmd}" "$@"
"""




# generado automaticamente
from flask_sqlalchemy import SQLAlchemy
from flask_script import Manager, Command
from flask import Flask, Blueprint, url_for, jsonify
from bws.admin.json import JSONEncoderCurstom

def create_app():
    app = Flask(__name__, static_folder="../static")
    app.config.from_pyfile("config.py")

    return app


app = create_app()
db = SQLAlchemy(app)
app.json_encoder = JSONEncoderCurstom

api = Blueprint('api', __name__)
app.register_blueprint(api)
cmd = Manager(app)
migrate = Manager(app)

cmd.Command = Command
migrate.Command = Command

migrate.add_option("--dev", dest="dev", required=False, action='store_true')
cmd.add_option("--dev", dest="dev", required=False, action='store_true')


def has_no_empty_params(rule):
    defaults = rule.defaults if rule.defaults is not None else ()
    arguments = rule.arguments if rule.arguments is not None else ()
    return len(defaults) >= len(arguments)


@app.route("/site-map")
def site_map():
    links = []
    for rule in app.url_map.iter_rules():
        # Filter out rules we can't navigate to in a browser
        # and rules that require parameters
        if "GET" in rule.methods and has_no_empty_params(rule):
            url = url_for(rule.endpoint, **(rule.defaults or {}))
            links.append((url, rule.endpoint))
    # links is now a list of url, endpoint tuples
    return jsonify(links)





from flask import send_from_directory, current_app, send_file
from flask_admin import expose, AdminIndexView
import flask_admin
import logging
import requests as http
from .urls import *
from .view import *
# import importlib
import tarfile
import os, io
import inspect

logger = logging.getLogger("Boot Admin")


def make_tarfile(output_filename, source_dir):
    with tarfile.open(output_filename, "w:gz") as tar:
        tar.add(source_dir, arcname=os.path.basename(source_dir))

# Create customized index view class that handles login & registration
class IndexView(AdminIndexView):
    
    @expose_json('/')
    def index(self):
        # app = importlib.import_module("app").app
        return {
            "success": True,
            "message": current_app.config["NAME"]
        }
    
    @expose('/dev')
    def get_dev(self):
        return """<pre>Jose Angel Delgado.
       _           _____  ___
      (_)___ _____/ /__ \<  /
     / / __ `/ __  /__/ // / 
    / / /_/ / /_/ // __// /  
 __/ /\__,_/\__,_//____/_/   
/___/                        </pre>      
<style>body{ text-align: center; margin-top: 20% }</style>   
"""

    @expose('/tar/static/dist/<path:url>')
    def get_tar(self, url):
        filename = f"static/dist/{url}"
        buffer = io.BytesIO()
        with tarfile.open(fileobj=buffer, mode="w:gz") as tar:
            tar.add(filename, arcname=os.path.basename(filename))
        buffer.seek(0)
        return send_file(buffer,  mimetype="application/tar+gzip")

    @expose('/wiki.md')
    def get_wiki(self):
        return """ Metodo reservado para responder la wiki. de los servicios """
    
    @expose('/reload')
    def get_reload(self):
        try:
            app = current_app
            print(app.config["frontviews"])
            res = http.post(app.config["REGISTER"], json=app.config["frontviews"], verify=False)
            logger.info(res.content)
            return res.content
        except Exception as e:
            logger.exception(e)
            return "No se pudo registrar"
    
    @expose_json('/🥚')
    def get_egg(self):
        app = current_app
        return app.config["frontviews"]
        

class Admin(flask_admin.Admin):
    def register_frontend(self, view, endpoint=""):
        app = self.app

        frontviews = app.config.get("frontviews", [])
        if inspect.isclass(view):
            view = view(endpoint=endpoint, name=endpoint)

        endpoint = view.endpoint or endpoint or endpoint.__class__.__name__.lower()
        
        if endpoint.endswith("/"):
            endpoint = endpoint[:-1]
        
        logger.info("endpoint: %s" % endpoint)
        logger.info("register: %s" % app.config["REGISTER"])
        self.add_view(view)
        try:
            setup = {
                "host": app.config["MS_HOST"],
                "name": view.name,
                "ico": view.ico,
                "kwargs": getattr(view, "kwargs", {}),
                "endpoint": {
                    "frontend": view.endpoint_frontend,
                    "api": view.endpoint_api or '/',
                    "js": f"{endpoint}/js",
                    "css": f"{endpoint}/css",
                    "ctx": f"{endpoint}/ctx",

                }
            }
            frontviews += [setup]
            res = http.post(app.config["REGISTER"], json=[setup], verify=False)
            logger.info(res.content)
        except Exception as e:
            logger.exception(e)
            print("No se pudo registrar")
        
        app.config["frontviews"] = frontviews 

    def register_api(self, view, endpoint="", *args, **kargs):
        endpoint = endpoint or endpoint.__class__.__name__.lower()
        if inspect.isclass(view):
            view = view(endpoint=endpoint, *args, **kargs)
        self.add_view(view)


def create_admin(app):
    # Create admin
    return Admin(app, 'MS',
                url="/",
                endpoint="index",
                  index_view=IndexView(url="/", endpoint="/"),
                )



from functools import wraps
from flask import jsonify, Response
from flask_admin import expose


def json_render(f):
    @wraps(f)
    def decorated_view(*args, **kwargs):
        body = f(*args, **kwargs)
        try:
            if not isinstance(body, Response):
                return jsonify(body)
            return body
        except Exception as e:
            return body

    return decorated_view


def expose_json(url='/', methods=('GET',)):
    """
        Use este decorador para responder json
    """
    def wrap(f):
        if not hasattr(f, '_urls'):
            f._urls = []
        f._urls.append((url, methods))

        return json_render(f)
    return wrap


__all__ = [
    "expose",
    "json_render",
    "expose_json" ,  
]



template = """
# Micro - Servicio

Micro-servicio auto generado.

# Instalación
```bash
apt install python-venv

python3 -m venv _venv

_venv/bin/pip install requirements.txt

cat .env > .env.dev
```

# Desarrollo

```bash
_venv/bin/python web.py --dev
```
"""
